@@ -7,8 +7,10 @@
 # directory of this source tree.
 
 from .freeform import FreeForm
+from ._linalg import eigvalsh, cond, norm, trace, slogdet
 from . import distributions
 
-__all__ = ['FreeForm', 'distributions']
+__all__ = ['FreeForm', 'distributions', 'eigvalsh', 'cond', 'norm', 'trace',
+           'slogdet']
 
 from .__version__ import __version__                          # noqa: F401 E402
