@@ -26,6 +26,11 @@ from ._plot_util import plot_fit, plot_density, plot_hilbert, plot_stieltjes
 from ._pade import fit_pade, eval_pade
 from ._decompress import decompress
 from ._sample import qmc_sample
+from ._support import detect_support
+
+# Fallback to previous numpy API
+if not hasattr(numpy, 'trapezoid'):
+    numpy.trapezoid = numpy.trapz
 
 __all__ = ['FreeForm']
 
@@ -50,17 +55,18 @@ class FreeForm(object):
         The support of the density of :math:`\\mathbf{A}`. If `None`, it is
         estimated from the minimum and maximum of the eigenvalues.
 
-    p : float, default=0.001
-        The edges of the support of the distribution is detected by the
-        :math:`p`-quantile on the left and :math:`(1-p)`-quantile on the right.
-        If the argument ``support`` is directly provided, this option is
-        ignored. This value should be between 0 and 1, ideally a small
-        number close to zero.
+    delta: float, default=1e-6
+        Size of perturbations into the upper half plane for Plemelj's
+        formula.
+
+    **kwargs : dict, optional
+        Parameters for the ``detect_support`` function can also be prescribed
+        here when ``support=None``.
 
     Notes
     -----
 
-    Notes.
+    TBD
 
     References
     ----------
@@ -73,12 +79,16 @@ class FreeForm(object):
     eig : numpy.array
         Eigenvalues of the matrix
 
+    support: tuple
+        The predicted (or given) support :math:`(\\lambda_{\\min},
+        \\lambda_{\\max})` of the eigenvalue density.
+
     psi : numpy.array
         Jacobi coefficients.
 
-    n   : int
-        Initial array size (assuming a square matrix when :math:`\\mathbf{A}`
-        is 2D)
+    n : int
+      Initial array size (assuming a square matrix when :math:`\\mathbf{A}` is
+      2D).
 
     Methods
     -------
@@ -98,6 +108,21 @@ class FreeForm(object):
     decompress
         Free decompression of spectral density
 
+    eigvalsh
+        Estimate the eigenvalues
+
+    cond
+        Estimate the condition number
+
+    trace
+        Estimate the trace of a matrix power
+
+    slogdet
+        Estimate the sign and logarithm of the determinant
+
+    norm
+        Estimate the Schatten norm
+
     Examples
     --------
 
@@ -110,13 +135,14 @@ class FreeForm(object):
     # init
     # ====
 
-    def __init__(self, A, support=None, p=0.001):
+    def __init__(self, A, support=None, delta=1e-6, **kwargs):
         """
         Initialization.
         """
 
         self.A = None
         self.eig = None
+        self.delta = delta    # Offset above real axis to apply Plemelj formula
 
         # Eigenvalues
         if A.ndim == 1:
@@ -134,52 +160,28 @@ class FreeForm(object):
 
         # Support
         if support is None:
-            self.lam_m, self.lam_p = self._detect_support(self.eig, p,
-                                                          smoothen=True)
+            self.lam_m, self.lam_p = detect_support(self.eig, **kwargs)
         else:
             self.lam_m = support[0]
             self.lam_p = support[1]
         self.support = (self.lam_m, self.lam_p)
 
         # Initialize
-        self.method = None
-        self.psi = None
-        self.alpha = None
-        self.beta = None
-        self._pade_sol = None
-
-    # ==============
-    # detect support
-    # ==============
-
-    def _detect_support(self, eig, p, smoothen=True):
-        """
-        """
-
-        # Using quantile directly.
-        if smoothen:
-            kde = gaussian_kde(eig)
-            xs = numpy.linspace(eig.min(), eig.max(), 1000)
-            fs = kde(xs)
-
-            cdf = numpy.cumsum(fs)
-            cdf /= cdf[-1]
-
-            lam_m = numpy.interp(p, cdf, xs)
-            lam_p = numpy.interp(1-p, cdf, xs)
-        else:
-            lam_m, lam_p = numpy.quantile(eig, [p, 1-p])
-
-        return lam_m, lam_p
+        self.method = None                 # fitting rho: jacobi, chebyshev
+        self.continuation = None           # analytic continuation: pade, wynn
+        self._pade_sol = None              # result of pade approximation
+        self.psi = None                    # coefficients of estimating rho
+        self.alpha = None                  # Jacobi polynomials alpha parameter
+        self.beta = None                   # Jacobi polynomials beta parameter
 
     # ===
     # fit
     # ===
 
     def fit(self, method='jacobi', K=10, alpha=0.0, beta=0.0, reg=0.0,
-            projection='gaussian', kernel_bw=None, damp=None, force=False,
-            pade_p=0, pade_q=1, odd_side='left', pade_reg=0.0, optimizer='ls',
-            plot=False, latex=False, save=False):
+            projection='gaussian', kernel_bw=0.001, damp=None, force=False,
+            continuation='pade', pade_p=0, pade_q=1, odd_side='left',
+            pade_reg=0.0, optimizer='ls', plot=False, latex=False, save=False):
         """
         Fit model to eigenvalues.
 
@@ -232,12 +234,22 @@ class FreeForm(object):
             If `True`, it forces the density to have unit mass and to be
             strictly positive.
 
+        continuation : {``'pade'``, ``wynn``}, default= ``'pade'``
+            Method of analytic continuation to construct the second branch of
+            Steltjes transform in the lower-half complex plane:
+
+            * ``'pade'``: using Reimann-Hilbert problem witth Pade
+              approximation.
+            * ``'wynn'``: Wynn's epsilon algorithm.
+
         pade_p : int, default=0
             Degree of polynomial :math:`P(z)` is :math:`q+p` where :math:`p`
-            can only be ``-1``, ``0``, or ``1``. See notes below.
+            can only be ``-1``, ``0``, or ``1``. See notes below. This option
+            is applicable if ``continuation='pade'``.
 
         pade_q : int, default=1
             Degree of polynomial :math:`Q(z)` is :math:`q`. See notes below.
+            This option is applicable if ``continuation='pade'``.
 
         odd_side : {``'left'``, ``'right'``}, default= ``'left'``
             In case of odd number of poles (when :math:`q` is odd), the extra
@@ -245,10 +257,12 @@ class FreeForm(object):
             while all other poles are split in half to the left and right. Note
             that this is only for the initialization of the poles. The
             optimizer will decide best location by moving them to the left or
-            right of the support.
+            right of the support. This option is applicable if
+            ``continuation='pade'``.
 
         pade_reg : float, default=0.0
-            Regularization for Pade approximation.
+            Regularization for Pade approximation. This option is applicable if
+            ``continuation='pade'``.
 
         optimizer : {``'ls'``, ``'de'``}, default= ``'ls'``
             Optimizer for Pade approximation, including:
@@ -256,9 +270,11 @@ class FreeForm(object):
             * ``'ls'``: least square (local, fast)
             * ``'de'``: differential evolution (global, slow)
 
+            This option is applicable if ``continuation='pade'``.
+
         plot : bool, default=False
             If `True`, the approximation coefficients and Pade approximation to
-            the Hilbert transform are plotted.
+            the Hilbert transform (if applicable) are plotted.
 
         latex : bool, default=False
             If `True`, the plot is rendered using LaTeX. This option is
@@ -331,7 +347,6 @@ class FreeForm(object):
                 #       adaptive=True)
                 # pdf = k.evaluate(xs)
 
-                # TEST
                 # import matplotlib.pyplot as plt
                 # plt.plot(xs, pdf)
                 # plt.grid(True)
@@ -364,7 +379,7 @@ class FreeForm(object):
                 psi = chebyshev_kernel_proj(xs, pdf, support=self.support,
                                             K=K, reg=reg)
         else:
-            raise ValueError('"method" is invalid.')
+            raise NotImplementedError('"method" is invalid.')
 
         # Damping
         if damp is not None:
@@ -403,34 +418,73 @@ class FreeForm(object):
         self.alpha = alpha
         self.beta = beta
 
-        # For holomorphic continuation for the lower half-plane
-        x_supp = numpy.linspace(self.lam_m, self.lam_p, 1000)
-        g_supp = 2.0 * numpy.pi * self.hilbert(x_supp)
+        # Analytic continuation
+        if continuation not in ['pade', 'wynn']:
+            raise NotImplementedError('"continuation" method is invalid.')
 
-        # Fit a pade approximation
-        # self._pade_sol = fit_pade(x_supp, g_supp, self.lam_m,
-        #                           self.lam_p, pade_p, pade_q, delta=1e-8,
-        #                           B=numpy.inf, S=numpy.inf)
-        self._pade_sol = fit_pade(x_supp, g_supp, self.lam_m, self.lam_p,
-                                  p=pade_p, q=pade_q, odd_side=odd_side,
-                                  pade_reg=pade_reg, safety=1.0, max_outer=40,
-                                  xtol=1e-12, ftol=1e-12, optimizer=optimizer,
-                                  verbose=0)
+        self.continuation = continuation
+
+        if self.continuation == 'pade':
+
+            # For holomorphic continuation for the lower half-plane
+            x_supp = numpy.linspace(self.lam_m, self.lam_p, 1000)
+            g_supp = 2.0 * numpy.pi * self.hilbert(x_supp)
+            self._pade_sol = fit_pade(x_supp, g_supp, self.lam_m, self.lam_p,
+                                      p=pade_p, q=pade_q, odd_side=odd_side,
+                                      pade_reg=pade_reg, safety=1.0,
+                                      max_outer=40, xtol=1e-12, ftol=1e-12,
+                                      optimizer=optimizer, verbose=0)
+        else:
+            # Do nothing. Make sure _pade_sol is still None
+            self._pade_sol = None
+
+            if method != 'chebyshev':
+                raise NotImplementedError(
+                    'Up to the current version, the analytic continuation ' +
+                    'using "wynn" is only implemented for "chebyshev" ' +
+                    'estimation method.')
 
         if plot:
-            g_supp_approx = eval_pade(x_supp[None, :], self._pade_sol)[0, :]
+            if self._pade_sol is not None:
+                g_supp_approx = eval_pade(x_supp[None, :],
+                                          self._pade_sol)[0, :]
+            else:
+                x_supp = None
+                g_supp = None
+                g_supp_approx = None
             plot_fit(psi, x_supp, g_supp, g_supp_approx, support=self.support,
                      latex=latex, save=save)
 
         return self.psi
 
+    # =============
+    # generate grid
+    # =============
+
+    def _generate_grid(self, scale, extend=1.0, N=500):
+        """
+        Generate a grid of points to evaluate density / Hilbert / Stieltjes
+        transforms.
+        """
+
+        radius = 0.5 * (self.lam_p - self.lam_m)
+        center = 0.5 * (self.lam_p + self.lam_m)
+
+        x_min = numpy.floor(extend * (center - extend * radius * scale))
+        x_max = numpy.ceil(extend * (center + extend * radius * scale))
+
+        x_min /= extend
+        x_max /= extend
+
+        return numpy.linspace(x_min, x_max, N)
+
     # =======
     # density
     # =======
 
     def density(self, x=None, plot=False, latex=False, save=False):
         """
-        Evaluate density.
+        Evaluate spectral density.
 
         Parameters
         ----------
@@ -471,16 +525,12 @@ class FreeForm(object):
         """
 
         if self.psi is None:
-            raise RuntimeError('"fit" the model first.')
+            raise RuntimeError('The model needs to be fit using the .fit() ' +
+                               'function.')
 
         # Create x if not given
         if x is None:
-            radius = 0.5 * (self.lam_p - self.lam_m)
-            center = 0.5 * (self.lam_p + self.lam_m)
-            scale = 1.25
-            x_min = numpy.floor(center - radius * scale)
-            x_max = numpy.ceil(center + radius * scale)
-            x = numpy.linspace(x_min, x_max, 500)
+            x = self._generate_grid(1.25)
 
         # Preallocate density to zero
         rho = numpy.zeros_like(x)
@@ -497,7 +547,7 @@ class FreeForm(object):
             raise RuntimeError('"method" is invalid.')
 
         # Check density is unit mass
-        mass = numpy.trapz(rho, x)
+        mass = numpy.trapezoid(rho, x)
         if not numpy.isclose(mass, 1.0, atol=1e-2):
             print(f'"rho" is not unit mass. mass: {mass:>0.3f}. Set ' +
                   r'"force=True".')
@@ -565,16 +615,12 @@ class FreeForm(object):
         """
 
         if self.psi is None:
-            raise RuntimeError('"fit" the model first.')
+            raise RuntimeError('The model needs to be fit using the .fit() ' +
+                               'function.')
 
         # Create x if not given
         if x is None:
-            radius = 0.5 * (self.lam_p - self.lam_m)
-            center = 0.5 * (self.lam_p + self.lam_m)
-            scale = 1.25
-            x_min = numpy.floor(center - radius * scale)
-            x_max = numpy.ceil(center + radius * scale)
-            x = numpy.linspace(x_min, x_max, 500)
+            x = self._generate_grid(1.25)
 
         # if (numpy.min(x) > self.lam_m) or (numpy.max(x) < self.lam_p):
         #     raise ValueError('"x" does not encompass support interval.')
@@ -599,7 +645,7 @@ class FreeForm(object):
 
         # Integrate each row over t using trapezoid rule on x_s
         # Namely, hilb[i] = int rho_s(t)/(t - x[i]) dt
-        hilb = numpy.trapz(D, x_s, axis=1) / numpy.pi
+        hilb = numpy.trapezoid(D, x_s, axis=1) / numpy.pi
 
         # We use negative sign convention
         hilb = -hilb
@@ -616,9 +662,20 @@ class FreeForm(object):
 
     def _glue(self, z):
         """
+        Glue function.
+
+        Notes
+        -----
+
+        This function needs self._pade_sol to be initialized in .fit()
+        function. This only works when continuation method is set to "pade".
         """
 
-        # Glue function
+        if self._pade_sol is None:
+            raise RuntimeError('"_glue" is called but "_pade_sol" is not' +
+                               'initialized. This is likely a ' +
+                               'development bug.')
+
         g = eval_pade(z, self._pade_sol)
 
         return g
@@ -629,8 +686,10 @@ class FreeForm(object):
 
     def stieltjes(self, x=None, y=None, plot=False, latex=False, save=False):
         """
-        Compute Stieltjes transform of the spectral density over a 2D Cartesian
-        grid on the complex plane.
+        Compute Stieltjes transform of the spectral density on a grid.
+
+        This function evaluates Stieltjes transform on an array of points, or
+        over a 2D Cartesian grid on the complex plane.
 
         Parameters
         ----------
@@ -689,53 +748,27 @@ class FreeForm(object):
         """
 
         if self.psi is None:
-            raise RuntimeError('"fit" the model first.')
+            raise RuntimeError('The model needs to be fit using the .fit() ' +
+                               'function.')
 
         # Create x if not given
         if x is None:
-            radius = 0.5 * (self.lam_p - self.lam_m)
-            center = 0.5 * (self.lam_p + self.lam_m)
-            scale = 2.0
-            x_min = numpy.floor(2.0 * (center - 2.0 * radius * scale)) / 2.0
-            x_max = numpy.ceil(2.0 * (center + 2.0 * radius * scale)) / 2.0
-            x = numpy.linspace(x_min, x_max, 500)
+            x = self._generate_grid(2.0, extend=2.0)
 
         # Create y if not given
-        if y is None:
-            y = numpy.linspace(-1, 1, 400)
+        if (plot is False) and (y is None):
+            # Do no tuse a Cartesian grid. Create a 1D array z slightly above
+            # the real line.
+            y = self.delta * 1j
+            z = x.astype(complex) + y             # shape (Nx,)
+        else:
+            # Use a Cartesian grid
+            if y is None:
+                y = numpy.linspace(-1, 1, 400)
+            x_grid, y_grid = numpy.meshgrid(x.real, y.real)
+            z = x_grid + 1j * y_grid              # shape (Ny, Nx)
 
-        x_grid, y_grid = numpy.meshgrid(x, y)
-        z = x_grid + 1j * y_grid              # shape (Ny, Nx)
-
-        # Set the number of bases as the number of x points insides support
-        mask_sup = numpy.logical_and(x >= self.lam_m, x <= self.lam_p)
-        n_base = 2 * numpy.sum(mask_sup)
-
-        # Stieltjes function
-        if self.method == 'jacobi':
-            stieltjes = partial(jacobi_stieltjes, psi=self.psi,
-                                support=self.support, alpha=self.alpha,
-                                beta=self.beta, n_base=n_base)
-        elif self.method == 'chebyshev':
-            stieltjes = partial(chebyshev_stieltjes, psi=self.psi,
-                                support=self.support)
-
-        mask_p = y >= 0.0
-        mask_m = y < 0.0
-
-        m1 = numpy.zeros_like(z)
-        m2 = numpy.zeros_like(z)
-
-        # Upper half-plane
-        m1[mask_p, :] = stieltjes(z[mask_p, :])
-
-        # Lower half-plane, use Schwarz reflection
-        m1[mask_m, :] = numpy.conjugate(
-            stieltjes(numpy.conjugate(z[mask_m, :])))
-
-        # Second Riemann sheet
-        m2[mask_p, :] = m1[mask_p, :]
-        m2[mask_m, :] = -m1[mask_m, :] + self._glue(z[mask_m, :])
+        m1, m2 = self._eval_stieltjes(z, branches=True)
 
         if plot:
             plot_stieltjes(x, y, m1, m2, self.support, latex=latex, save=save)
@@ -746,7 +779,7 @@ class FreeForm(object):
     # eval stieltjes
     # ==============
 
-    def _eval_stieltjes(self, z):
+    def _eval_stieltjes(self, z, branches=False):
         """
         Compute Stieltjes transform of the spectral density.
 
@@ -757,56 +790,53 @@ class FreeForm(object):
             The z values in the complex plan where the Stieltjes transform is
             evaluated.
 
+        branches : bool, default = False
+            Return both the principal and secondary branches of the Stieltjes
+            transform. The default ``branches=False`` will return only
+            the secondary branch.
 
         Returns
         -------
 
         m_p : numpy.ndarray
-            The Stieltjes transform on the principal branch.
+            The Stieltjes transform on the principal branch if
+            ``branches=True``.
 
         m_m : numpy.ndarray
             The Stieltjes transform continued to the secondary branch.
-
-        See Also
-        --------
-        density
-        hilbert
-
-        Notes
-        -----
-
-        Notes.
-
-        References
-        ----------
-
-        .. [1] tbd
-
-        Examples
-        --------
-
-        .. code-block:: python
-
-            >>> from freealg import FreeForm
         """
 
         if self.psi is None:
             raise RuntimeError('"fit" the model first.')
 
+        # Allow for arbitrary input shapes
         z = numpy.asarray(z)
         shape = z.shape
         if len(shape) == 0:
             shape = (1,)
         z = z.reshape(-1, 1)
 
+        # # Set the number of bases as the number of x points insides support
+        # mask_sup = numpy.logical_and(z.real >= self.lam_m,
+        #                              z.real <= self.lam_p)
+        # n_base = 2 * numpy.sum(mask_sup)
+
         # Stieltjes function
         if self.method == 'jacobi':
             stieltjes = partial(jacobi_stieltjes, psi=self.psi,
                                 support=self.support, alpha=self.alpha,
-                                beta=self.beta)
+                                beta=self.beta)  # n_base = n_base
+
         elif self.method == 'chebyshev':
+
+            if self.continuation == 'wynn':
+                use_wynn_epsilon = True
+            else:
+                use_wynn_epsilon = False
+
             stieltjes = partial(chebyshev_stieltjes, psi=self.psi,
-                                support=self.support)
+                                support=self.support,
+                                use_wynn_epsilon=use_wynn_epsilon)
 
         mask_p = z.imag >= 0.0
         mask_m = z.imag < 0.0
@@ -814,48 +844,63 @@ class FreeForm(object):
         m1 = numpy.zeros_like(z)
         m2 = numpy.zeros_like(z)
 
-        # Upper half-plane
-        m1[mask_p] = stieltjes(z[mask_p].reshape(-1, 1)).reshape(-1)
+        if self.continuation == 'pade':
+            # Upper half-plane
+            m1[mask_p] = stieltjes(z[mask_p].reshape(-1, 1)).ravel()
 
-        # Lower half-plane, use Schwarz reflection
-        m1[mask_m] = numpy.conjugate(
-            stieltjes(numpy.conjugate(z[mask_m].reshape(-1, 1)))).reshape(-1)
+            # Lower half-plane, use Schwarz reflection
+            z_conj = numpy.conjugate(z[mask_m].reshape(-1, 1))
+            m1[mask_m] = numpy.conjugate(stieltjes(z_conj)).ravel()
 
-        # Second Riemann sheet
-        m2[mask_p] = m1[mask_p]
-        m2[mask_m] = -m1[mask_m] + self._glue(
-            z[mask_m].reshape(-1, 1)).reshape(-1)
+            # Second Riemann sheet
+            m2[mask_p] = m1[mask_p]
+            m2[mask_m] = -m1[mask_m] + self._glue(
+                z[mask_m].reshape(-1, 1)).ravel()
 
-        m1, m2 = m1.reshape(*shape), m2.reshape(*shape)
+        elif self.continuation == 'wynn':
+            m2[:] = stieltjes(z.reshape(-1, 1)).reshape(*m2.shape)
+            if branches:
+                m1[mask_p] = m2[mask_p]
+                z_conj = numpy.conjugate(z[mask_m].reshape(-1, 1))
+                m1[mask_m] = numpy.conjugate(stieltjes(z_conj)).ravel()
 
-        return m1, m2
+        else:
+            raise NotImplementedError('Invalid continuation method.')
+
+        if not branches:
+            return m2.reshape(*shape)
+        else:
+            m1 = m1.reshape(*shape)
+            m2 = m2.reshape(*shape)
+            return m1, m2
 
     # ==========
     # decompress
     # ==========
 
-    def decompress(self, size, x=None, delta=1e-6, iterations=500,
-                   step_size=0.1, tolerance=1e-4, seed=None, plot=False,
-                   latex=False, save=False):
+    def decompress(self, size, x=None, method='newton', max_iter=500,
+                   step_size=0.1, tolerance=1e-4, plot=False, latex=False,
+                   save=False, plot_diagnostics=False):
         """
         Free decompression of spectral density.
 
         Parameters
         ----------
 
-        size : int
-            Size of the decompressed matrix.
+        size : int or array_like
+            Size(s) of the decompressed matrix. This can be a scalar or an
+            array of sizes. For each matrix size in ``size`` array, a density
+            is produced.
 
         x : numpy.array, default=None
             Positions where density to be evaluated at. If `None`, an interval
             slightly larger than the support interval will be used.
 
-        delta: float, default=1e-4
-            Size of the perturbation into the upper half plane for Plemelj's
-            formula.
+        method : {``'newton'``, ``'secant'``}, default= ``'newton'``
+            Root-finding method.
 
-        iterations: int, default=500
-            Maximum number of Newton iterations.
+        max_iter: int, default=500
+            Maximum number of root-finding method iterations.
 
         step_size: float, default=0.1
             Step size for Newton iterations.
@@ -864,9 +909,6 @@ class FreeForm(object):
             Tolerance for the solution obtained by the Newton solver. Also
             used for the finite difference approximation to the derivative.
 
-        seed : int, default=None
-            Seed for random number generator. Used for QMC sampling.
-
         plot : bool, default=False
             If `True`, density is plotted.
 
@@ -879,15 +921,25 @@ class FreeForm(object):
             assumed to the save filename (with the file extension). This option
             is relevant only if ``plot=True``.
 
+        plot_diagnostics : bool, default=False
+            Plots diagnostics including convergence and number of iterations
+            of root finding method.
+
         Returns
         -------
 
-        rho : numpy.array
-            Spectral density
+        rho : numpy.array or numpy.ndarray
+            Estimated spectral density at locations x. ``rho`` can be a 1D or
+            2D array output:
 
-        eigs : numpy.array
-            Estimated eigenvalues as low-discrepancy samples of the estimated
-            spectral density.
+            * If ``size`` is a scalar, ``rho`` is a 1D array od the same size
+              as ``x``.
+            * If ``size`` is an array of size `n`, ``rho`` is a 2D array with
+              `n` rows, where each row corresponds to decompression to a size.
+              Number of columns of ``rho`` is the same as the size of ``x``.
+
+        x : numpy.array
+            Locations where the spectral density is estimated
 
         See Also
         --------
@@ -913,17 +965,401 @@ class FreeForm(object):
             >>> from freealg import FreeForm
         """
 
-        size = int(size)
+        # Check size argument
+        if numpy.isscalar(size):
+            size = int(size)
+        else:
+            # Check monotonic increment (either all increasing or decreasing)
+            diff = numpy.diff(size)
+            if not (numpy.all(diff >= 0) or numpy.all(diff <= 0)):
+                raise ValueError('"size" increment should be monotonic.')
 
-        rho, x, (lb, ub) = decompress(self, size, x=x, delta=delta,
-                                      iterations=iterations,
-                                      step_size=step_size, tolerance=tolerance)
-        x, rho = x.ravel(), rho.ravel()
+        # Decompression ratio equal to e^{t}.
+        alpha = numpy.atleast_1d(size) / self.n
 
+        # Lower and upper bound on new support
+        m = self._eval_stieltjes
+        hilb_lb = (1.0 / m(self.lam_m + self.delta * 1j).item()).real
+        hilb_ub = (1.0 / m(self.lam_p + self.delta * 1j).item()).real
+        lb = self.lam_m - (numpy.max(alpha) - 1) * hilb_lb
+        ub = self.lam_p - (numpy.max(alpha) - 1) * hilb_ub
+
+        # Create x if not given
+        if x is None:
+            radius = 0.5 * (ub - lb)
+            center = 0.5 * (ub + lb)
+            scale = 1.25
+            x_min = numpy.floor(center - radius * scale)
+            x_max = numpy.ceil(center + radius * scale)
+            x = numpy.linspace(x_min, x_max, 500)
+        else:
+            x = numpy.asarray(x)
+
+        if (alpha.size > 8) and plot_diagnostics:
+            raise RuntimeError(
+                'Too many diagnostic plots for %d sizes.' % alpha.size)
+
+        # Decompress to each alpha
+        rho = numpy.zeros((alpha.size, x.size), dtype=float)
+        for i in range(alpha.size):
+
+            # Initial guess for roots (only for the first iteration)
+            # if i == 0:
+            #     roots = numpy.full(x.shape, numpy.mean(self.support) - 0.1j,
+            #                        dtype=numpy.complex128)
+            roots = None
+
+            rho[i, :], roots = decompress(
+                self, alpha[i], x, roots_init=roots, method=method,
+                delta=self.delta, max_iter=max_iter, step_size=step_size,
+                tolerance=tolerance, plot_diagnostics=plot_diagnostics)
+
+        # If the input size was only a scalar, return a 1D rho, otherwise 2D.
+        if numpy.isscalar(size):
+            rho = numpy.squeeze(rho)
+
+        # Plot only the last size
         if plot:
-            plot_density(x, rho, support=(lb, ub),
+            if numpy.isscalar(size):
+                rho_last = rho
+            else:
+                rho_last = rho[-1, :]
+            plot_density(x, rho_last, support=(lb, ub),
                          label='Decompression', latex=latex, save=save)
 
+        return rho, x
+
+    # ========
+    # eigvalsh
+    # ========
+
+    def eigvalsh(self, size=None, seed=None, **kwargs):
+        """
+        Estimate the eigenvalues.
+
+        This function estimates the eigenvalues of the freeform matrix
+        or a larger matrix containing it using free decompression.
+
+        Parameters
+        ----------
+
+        size : int, default=None
+            The size of the matrix containing :math:`\\mathbf{A}` to estimate
+            eigenvalues of. If None, returns estimates of the eigenvalues of
+            :math:`\\mathbf{A}` itself.
+
+        seed : int, default=None
+            The seed for the Quasi-Monte Carlo sampler.
+
+        **kwargs : dict, optional
+            Pass additional options to the underlying
+            :func:`FreeForm.decompress` function.
+
+        Returns
+        -------
+
+        eigs : numpy.array
+            Eigenvalues of decompressed matrix
+
+        See Also
+        --------
+
+        FreeForm.decompress
+        FreeForm.cond
+
+        Notes
+        -----
+
+        All arguments to the `.decompress()` procedure can be provided.
+
+        Examples
+        --------
+
+        .. code-block:: python
+            :emphasize-lines: 1
+
+            >>> from freealg import FreeForm
+        """
+
+        if size is None:
+            size = self.n
+
+        rho, x = self.decompress(size, **kwargs)
         eigs = numpy.sort(qmc_sample(x, rho, size, seed=seed))
 
-        return rho, eigs
+        return eigs
+
+    # ====
+    # cond
+    # ====
+
+    def cond(self, size=None, seed=None, **kwargs):
+        """
+        Estimate the condition number.
+
+        This function estimates the condition number of the matrix
+        :math:`\\mathbf{A}` or a larger matrix containing :math:`\\mathbf{A}`
+        using free decompression.
+
+        Parameters
+        ----------
+
+        size : int, default=None
+            The size of the matrix containing :math:`\\mathbf{A}` to estimate
+            eigenvalues of. If None, returns estimates of the eigenvalues of
+            :math:`\\mathbf{A}` itself.
+
+        **kwargs : dict, optional
+            Pass additional options to the underlying
+            :func:`FreeForm.decompress` function.
+
+        Returns
+        -------
+
+        c : float
+            Condition number
+
+        See Also
+        --------
+
+        FreeForm.eigvalsh
+        FreeForm.norm
+        FreeForm.slogdet
+        FreeForm.trace
+
+        Examples
+        --------
+
+        .. code-block:: python
+            :emphasize-lines: 1
+
+            >>> from freealg import FreeForm
+        """
+
+        eigs = self.eigvalsh(size=size, **kwargs)
+        return eigs.max() / eigs.min()
+
+    # =====
+    # trace
+    # =====
+
+    def trace(self, size=None, p=1.0, seed=None, **kwargs):
+        """
+        Estimate the trace of a power.
+
+        This function estimates the trace of the matrix power
+        :math:`\\mathbf{A}^p` of the freeform or that of a larger matrix
+        containing it.
+
+        Parameters
+        ----------
+
+        size : int, default=None
+            The size of the matrix containing :math:`\\mathbf{A}` to estimate
+            eigenvalues of. If None, returns estimates of the eigenvalues of
+            :math:`\\mathbf{A}` itself.
+
+        p : float, default=1.0
+            The exponent :math:`p` in :math:`\\mathbf{A}^p`.
+
+        seed : int, default=None
+            The seed for the Quasi-Monte Carlo sampler.
+
+        **kwargs : dict, optional
+            Pass additional options to the underlying
+            :func:`FreeForm.decompress` function.
+
+        Returns
+        -------
+
+        trace : float
+            matrix trace
+
+        See Also
+        --------
+
+        FreeForm.eigvalsh
+        FreeForm.cond
+        FreeForm.slogdet
+        FreeForm.norm
+
+        Notes
+        -----
+
+        The trace is highly amenable to subsampling: under free decompression
+        the average eigenvalue is assumed constant, so the trace increases
+        linearly. Traces of powers fall back to :func:`eigvalsh`.
+        All arguments to the `.decompress()` procedure can be provided.
+
+        Examples
+        --------
+
+        .. code-block:: python
+            :emphasize-lines: 1
+
+            >>> from freealg import FreeForm
+        """
+
+        if numpy.isclose(p, 1.0):
+            return numpy.mean(self.eig) * (size / self.n)
+
+        eig = self.eigvalsh(size=size, seed=seed, **kwargs)
+        return numpy.sum(eig ** p)
+
+    # =======
+    # slogdet
+    # =======
+
+    def slogdet(self, size=None, seed=None, **kwargs):
+        """
+        Estimate the sign and logarithm of the determinant.
+
+        This function estimates the *slogdet* of the freeform or that of
+        a larger matrix containing it using free decompression.
+
+        Parameters
+        ----------
+
+        size : int, default=None
+            The size of the matrix containing :math:`\\mathbf{A}` to estimate
+            eigenvalues of. If None, returns estimates of the eigenvalues of
+            :math:`\\mathbf{A}` itself.
+
+        seed : int, default=None
+            The seed for the Quasi-Monte Carlo sampler.
+
+        Returns
+        -------
+
+        sign : float
+            Sign of determinant
+
+        ld : float
+            natural logarithm of the absolute value of the determinant
+
+        See Also
+        --------
+
+        FreeForm.eigvalsh
+        FreeForm.cond
+        FreeForm.trace
+        FreeForm.norm
+
+        Notes
+        -----
+
+        All arguments to the `.decompress()` procedure can be provided.
+
+        Examples
+        --------
+
+        .. code-block:: python
+            :emphasize-lines: 1
+
+            >>> from freealg import FreeForm
+        """
+
+        eigs = self.eigvalsh(size=size, seed=seed, **kwargs)
+        sign = numpy.prod(numpy.sign(eigs))
+        ld = numpy.sum(numpy.log(numpy.abs(eigs)))
+        return sign, ld
+
+    # ====
+    # norm
+    # ====
+
+    def norm(self, size=None, order=2, seed=None, **kwargs):
+        """
+        Estimate the Schatten norm.
+
+        This function estimates the norm of the freeform or a larger
+        matrix containing it using free decompression.
+
+        Parameters
+        ----------
+
+        size : int, default=None
+            The size of the matrix containing :math:`\\mathbf{A}` to estimate
+            eigenvalues of. If None, returns estimates of the eigenvalues of
+            :math:`\\mathbf{A}` itself.
+
+        order : {float, ``''inf``, ``'-inf'``, ``'fro'``, ``'nuc'``}, default=2
+            Order of the norm.
+
+            * float :math:`p`: Schatten p-norm.
+            * ``'inf'``: Largest absolute eigenvalue
+              :math:`\\max \\vert \\lambda_i \\vert)`
+            * ``'-inf'``: Smallest absolute eigenvalue
+              :math:`\\min \\vert \\lambda_i \\vert)`
+            * ``'fro'``: Frobenius norm corresponding to :math:`p=2`
+            * ``'nuc'``: Nuclear (or trace) norm corresponding to :math:`p=1`
+
+        seed : int, default=None
+            The seed for the Quasi-Monte Carlo sampler.
+
+        **kwargs : dict, optional
+            Pass additional options to the underlying
+            :func:`FreeForm.decompress` function.
+
+        Returns
+        -------
+
+        norm : float
+            matrix norm
+
+        See Also
+        --------
+
+        FreeForm.eigvalsh
+        FreeForm.cond
+        FreeForm.slogdet
+        FreeForm.trace
+
+        Notes
+        -----
+
+        Thes Schatten :math:`p`-norm is defined by
+
+        .. math::
+
+            \\Vert \\mathbf{A} \\Vert_p = \\left(
+            \\sum_{i=1}^N \\vert \\lambda_i \\vert^p \\right)^{1/p}.
+
+        Examples
+        --------
+
+        .. code-block:: python
+            :emphasize-lines: 1
+
+            >>> from freealg import FreeForm
+        """
+
+        eigs = self.eigvalsh(size, seed=seed, **kwargs)
+
+        # Check order type and convert to float
+        if order == 'nuc':
+            order = 1
+        elif order == 'fro':
+            order = 2
+        elif order == 'inf':
+            order = float('inf')
+        elif order == '-inf':
+            order = -float('inf')
+        elif not isinstance(order,
+                            (int, float, numpy.integer, numpy.floating)) \
+                and not isinstance(order, (bool, numpy.bool_)):
+            raise ValueError('"order" is invalid.')
+
+        # Compute norm
+        if numpy.isinf(order) and not numpy.isneginf(order):
+            norm_ = max(numpy.abs(eigs))
+
+        elif numpy.isneginf(order):
+            norm_ = min(numpy.abs(eigs))
+
+        elif isinstance(order, (int, float, numpy.integer, numpy.floating)) \
+                and not isinstance(order, (bool, numpy.bool_)):
+            norm_q = numpy.sum(numpy.abs(eigs)**order)
+            norm_ = norm_q**(1.0 / order)
+
+        return norm_
